package com.atguigu.gateway.filter;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.atguigu.constant.MessageConstant;
import com.atguigu.spzx.model.entity.user.UserInfo;
import com.atguigu.spzx.model.vo.result.Result;
import com.atguigu.spzx.model.vo.result.ResultCodeEnum;
import com.atguigu.utils.EmptyUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.Ordered;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.util.AntPathMatcher;
import org.springframework.util.StringUtils;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;
import java.util.List;

@Configuration
public class AuthGlobalFilter implements GlobalFilter, Ordered {
    private final AntPathMatcher antPathMatcher = new AntPathMatcher();
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    @Autowired
    private EmptyUtil emptyUtil;

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        ServerHttpRequest request = exchange.getRequest();
        String path = request.getURI().getPath();
        // 判断请求路径
        if (antPathMatcher.match("/api/**/auth/**", path)) {
            UserInfo userInfo = getUserinfo(request);
            emptyUtil.isEmpty(userInfo, MessageConstant.USER_DOES_NOT_EXIST);
            ServerHttpResponse response = exchange.getResponse();
            return out(exchange.getResponse());

        }
        return chain.filter(exchange);
    }

    private UserInfo getUserinfo(ServerHttpRequest request) {
        List<String> tokenList = request.getHeaders().get("token");
        emptyUtil.isEmpty(tokenList, MessageConstant.TOKEN_IS_EMPTY);
        String token = tokenList.get(0);
        if (StringUtils.hasText(token)) {
            String userJson = (String) redisTemplate.opsForValue().get(token);
            emptyUtil.isEmpty(userJson, MessageConstant.TOKEN_IS_EMPTY);
            return JSON.parseObject(userJson, UserInfo.class);
        }
        return null;
    }

    @Override
    public int getOrder() {
        return 0;
    }

    private Mono<Void> out(ServerHttpResponse response) {
        Result<ResultCodeEnum> result = Result.build(null, ResultCodeEnum.LOGIN_AUTH);
        byte[] bits = JSONObject.toJSONString(result).getBytes(StandardCharsets.UTF_8);
        DataBuffer buffer = response.bufferFactory().wrap(bits);
        // 指定编码，否则在浏览器中会中文乱码
        response.getHeaders().add("Content-Type", "application/json;charset=UTF-8");
        return response.writeWith(Mono.just(buffer));
    }
}
